@@ -1,10 +1,11 @@
 //! Core types and encoding operations used by the age file format.
 
+use base64::{prelude::BASE64_STANDARD_NO_PAD, Engine};
 use rand::{
     distributions::{Distribution, Uniform},
     thread_rng, RngCore,
 };
-use secrecy::{ExposeSecret, Secret};
+use secrecy::{ExposeSecret, ExposeSecretMut, SecretBox};
 
 /// The prefix identifying an age stanza.
 const STANZA_TAG: &str = "-> ";
@@ -13,11 +14,26 @@ const STANZA_TAG: &str = "-> ";
 pub const FILE_KEY_BYTES: usize = 16;
 
 /// A file key for encrypting or decrypting an age file.
-pub struct FileKey(Secret<[u8; FILE_KEY_BYTES]>);
+pub struct FileKey(SecretBox<[u8; FILE_KEY_BYTES]>);
 
-impl From<[u8; FILE_KEY_BYTES]> for FileKey {
-    fn from(file_key: [u8; FILE_KEY_BYTES]) -> Self {
-        FileKey(Secret::new(file_key))
+impl FileKey {
+    /// Creates a file key using a pre-boxed key.
+    pub fn new(file_key: Box<[u8; FILE_KEY_BYTES]>) -> Self {
+        Self(SecretBox::new(file_key))
+    }
+
+    /// Creates a file key using a function that can initialize the key in-place.
+    pub fn init_with_mut(ctr: impl FnOnce(&mut [u8; FILE_KEY_BYTES])) -> Self {
+        Self(SecretBox::init_with_mut(ctr))
+    }
+
+    /// Same as [`Self::init_with_mut`], but the constructor can be fallible.
+    pub fn try_init_with_mut<E>(
+        ctr: impl FnOnce(&mut [u8; FILE_KEY_BYTES]) -> Result<(), E>,
+    ) -> Result<Self, E> {
+        let mut file_key = SecretBox::new(Box::new([0; FILE_KEY_BYTES]));
+        ctr(file_key.expose_secret_mut())?;
+        Ok(Self(file_key))
     }
 }
 
@@ -60,7 +76,7 @@ impl<'a> AgeStanza<'a> {
         data[full_chunks.len() * 64..].copy_from_slice(partial_chunk);
 
         // The chunks are guaranteed to contain Base64 characters by construction.
-        base64::decode_config(&data, base64::STANDARD_NO_PAD).unwrap()
+        BASE64_STANDARD_NO_PAD.decode(&data).unwrap()
     }
 }
 
@@ -68,7 +84,7 @@ impl<'a> AgeStanza<'a> {
 /// recipient.
 ///
 /// This is the owned type; see [`AgeStanza`] for the reference type.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq)]
 pub struct Stanza {
     /// A tag identifying this stanza type.
     pub tag: String,
@@ -89,6 +105,16 @@ impl From<AgeStanza<'_>> for Stanza {
     }
 }
 
+/// Checks whether the string is a valid age "arbitrary string" (`1*VCHAR` in ABNF).
+pub fn is_arbitrary_string<S: AsRef<str>>(s: &S) -> bool {
+    let s = s.as_ref();
+    !s.is_empty()
+        && s.chars().all(|c| match u8::try_from(c) {
+            Ok(u) => (33..=126).contains(&u),
+            Err(_) => false,
+        })
+}
+
 /// Creates a random recipient stanza that exercises the joint in the age v1 format.
 ///
 /// This function is guaranteed to return a valid stanza, but makes no other guarantees
@@ -324,6 +350,7 @@ pub mod read {
 
 /// Encoding operations for age types.
 pub mod write {
+    use base64::{prelude::BASE64_STANDARD_NO_PAD, Engine};
     use cookie_factory::{
         combinator::string,
         multi::separated_list,
@@ -336,7 +363,7 @@ pub mod write {
     use super::STANZA_TAG;
 
     fn wrapped_encoded_data<'a, W: 'a + Write>(data: &[u8]) -> impl SerializeFn<W> + 'a {
-        let encoded = base64::encode_config(data, base64::STANDARD_NO_PAD);
+        let encoded = BASE64_STANDARD_NO_PAD.encode(data);
 
         move |mut w: WriteContext<W>| {
             let mut s = encoded.as_str();
@@ -377,6 +404,7 @@ pub mod write {
 
 #[cfg(test)]
 mod tests {
+    use base64::{prelude::BASE64_STANDARD_NO_PAD, Engine};
     use nom::error::ErrorKind;
 
     use super::{read, write};
@@ -385,11 +413,9 @@ mod tests {
     fn parse_age_stanza() {
         let test_tag = "X25519";
         let test_args = &["CJM36AHmTbdHSuOQL+NESqyVQE75f2e610iRdLPEN20"];
-        let test_body = base64::decode_config(
-            "C3ZAeY64NXS4QFrksLm3EGz+uPRyI0eQsWw7LWbbYig",
-            base64::STANDARD_NO_PAD,
-        )
-        .unwrap();
+        let test_body = BASE64_STANDARD_NO_PAD
+            .decode("C3ZAeY64NXS4QFrksLm3EGz+uPRyI0eQsWw7LWbbYig")
+            .unwrap();
 
         // The only body line is short, so we don't need a trailing empty line.
         let test_stanza = "-> X25519 CJM36AHmTbdHSuOQL+NESqyVQE75f2e610iRdLPEN20
@@ -433,11 +459,9 @@ C3ZAeY64NXS4QFrksLm3EGz+uPRyI0eQsWw7LWbbYig
     fn age_stanza_with_full_body() {
         let test_tag = "full-body";
         let test_args = &["some", "arguments"];
-        let test_body = base64::decode_config(
-            "xD7o4VEOu1t7KZQ1gDgq2FPzBEeSRqbnqvQEXdLRYy143BxR6oFxsUUJCRB0ErXA",
-            base64::STANDARD_NO_PAD,
-        )
-        .unwrap();
+        let test_body = BASE64_STANDARD_NO_PAD
+            .decode("xD7o4VEOu1t7KZQ1gDgq2FPzBEeSRqbnqvQEXdLRYy143BxR6oFxsUUJCRB0ErXA")
+            .unwrap();
 
         // The body fills a complete line, so it requires a trailing empty line.
         let test_stanza = "-> full-body some arguments
@@ -460,11 +484,9 @@ xD7o4VEOu1t7KZQ1gDgq2FPzBEeSRqbnqvQEXdLRYy143BxR6oFxsUUJCRB0ErXA
     fn age_stanza_with_legacy_full_body() {
         let test_tag = "full-body";
         let test_args = &["some", "arguments"];
-        let test_body = base64::decode_config(
-            "xD7o4VEOu1t7KZQ1gDgq2FPzBEeSRqbnqvQEXdLRYy143BxR6oFxsUUJCRB0ErXA",
-            base64::STANDARD_NO_PAD,
-        )
-        .unwrap();
+        let test_body = BASE64_STANDARD_NO_PAD
+            .decode("xD7o4VEOu1t7KZQ1gDgq2FPzBEeSRqbnqvQEXdLRYy143BxR6oFxsUUJCRB0ErXA")
+            .unwrap();
 
         // The body fills a complete line, but lacks a trailing empty line.
         let test_stanza = "-> full-body some arguments
